<?xml version="1.0" encoding="iso-8859-1"?> 
<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:saxon="http://saxon.sf.net/"
    xmlns:eg="http://www.example.org/">

<!-- Stylesheet parameters:  -->
<!-- env specifies the environment where we run the stylesheet -->
<xsl:param name="env" select="'final'"/>
<!-- images=yes turns image generation on -->
<xsl:param name="images" select="'no'"/>

<!-- Master document: -->   
<xsl:variable name="master" select="document('_master.xml')"/>

<!-- Are we transforming the master document 
     (e.g., in batch processing)? -->   
<xsl:variable name="this-master" select="
    boolean(/*[name()='site'])"/>

<!-- Source path for this environment: -->
<xsl:variable name="src-path" select="
    $master//environment[@id=$env]/src-path"/>

<!-- Path to the output directory: -->
<xsl:variable name="out-path" select="
    $master//environment[@id=$env]/out-path"/>

<!-- Target path, for src and href attributes in HTML: -->
<xsl:variable name="target-path" select="
    $master//environment[@id=$env]/target-path"/>

<!-- Image output directory, for accessing and generating images
     during transformation: -->
<xsl:variable name="out-img">
  <xsl:value-of select="$out-path"/>
  <xsl:value-of select="$master//environment[@id = $env]/img-path"/>
  <xsl:text>/</xsl:text>
</xsl:variable>

<!-- Image target directory, for image src attributes in HTML: -->
<xsl:variable name="target-img">
  <xsl:value-of select="$target-path"/>
  <xsl:value-of select="$master//environment[@id = $env]/img-path"/>
  <xsl:text>/</xsl:text>
</xsl:variable>

<!-- Filename extensions: -->
<xsl:variable name="src-ext">.xml</xsl:variable>
<xsl:variable name="out-ext">.html</xsl:variable>

<!-- Current language, based on the current directory: -->
<xsl:variable name="lang">
  <xsl:choose>
    <xsl:when test="
        substring-after(saxon:systemId(), $src-path) = ''">
      <xsl:message terminate="yes">
        Error: Source file path does not match $src-path 
        $env = <xsl:value-of select="$env"/>
        systemId = <xsl:value-of select="saxon:systemId()"/>	
      </xsl:message>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="
          substring-before(
            substring-after(saxon:systemId(), $src-path),
            '/')"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- Path to the current file (without extension) 
     starting from the root dir of the current language: -->
<xsl:variable name="current">
  <xsl:choose>
    <xsl:when test="
        not($master//languages/lang = $lang) 
        and not($this-master)">
      <xsl:message terminate="yes">
        Error: Not in a valid language directory 
        $lang = <xsl:value-of select="$lang"/> 
      </xsl:message>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="
          substring-before(
            substring-after(
              substring-after(saxon:systemId(), $src-path), 
              concat($lang, '/')),
            $src-ext)"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- True if the current page is the site's front page: -->
<xsl:variable name="frontpage" select="$current = 'index'"/>

<!-- Deabbreviation functions: -->

<!-- Deabbreviate external links -->
<xsl:function name="eg:ext-link">
  <xsl:param name="abbr"/>
  <xsl:value-of select="
      if (starts-with ($abbr, 'http://') or 
          starts-with ($abbr, 'ftp://')) 
        then $abbr
        else concat('http://', $abbr)"/>
</xsl:function>

<!-- Convert a file:/ URI into Windows native format -->
<xsl:function name="eg:os-path">
  <xsl:param name="s"/>
  <xsl:value-of select="
      if (starts-with($s, 'file:/')) 
        then substring-after($s, 'file:/')
        else $s"/>
</xsl:function>

<!-- Return pathname of a page document -->
<xsl:function name="eg:page-src">
  <xsl:param name="abbr"/>
  <xsl:param name="lang"/>
  <xsl:value-of select="
      concat(
        $src-path, 
        $lang, '/',
        $master//(page|block)[(@id,tokenize(@alias, '\s+')) = $abbr]
          /@src,
        $src-ext)"/>
</xsl:function>

<!-- Return URI of a transformed HTML page -->
<xsl:function name="eg:page-link">
  <xsl:param name="path"/>
  <xsl:param name="abbr"/>
  <xsl:param name="lang"/>
  <xsl:value-of select="
      concat(
        $path, 
        $lang, '/',
        $master//(page|block)[(@id,tokenize(@alias, '\s+')) = $abbr]
          /@src,
        $out-ext)"/>
</xsl:function>

</xsl:stylesheet>